<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';

require_once 'lanet_group.php';
require_once 'lanet_channel.php';

///////////////////////////////////////////////////////////////////////////

class LanetTv extends AbstractTv
{
    public function __construct()
    {
        parent::__construct(
            AbstractTv::MODE_CHANNELS_N_TO_M,
            true,
            false,
            LanetConfig::ALL_CHANNEL_GROUP_ID);
    }
    
    public function get_fav_icon_url()
    {
        return LanetConfig::FAV_CHANNEL_GROUP_ICON_PATH;
    }
    
    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    protected function load_channels(&$plugin_cookies)
    {
        try
        {
            $doc = HD::http_get_document(LanetConfig::CHANNEL_LIST_URL);
            TRACE('Received channels list: chars:'.strlen($doc));
        }
        catch (Exception $e)
        {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }
     
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }
        
        if ($xml->getName() !== 'tv_info')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'tv_info'");
            throw new Exception('Invalid XML document');
        }
        
        $this->channels = new HashedArray();
        $this->groups = new HashedArray();

        if ($this->is_favorites_supported())
        {
            hd_print('Favourites supported. Add Favourites group');
            $this->groups->put(
                new FavoritesGroup(
                    $this,
                    LanetConfig::FAV_CHANNEL_GROUP_ID,
                    LanetConfig::FAV_CHANNEL_GROUP_CAPTION,
                    LanetConfig::FAV_CHANNEL_GROUP_ICON_PATH));
        }

        TRACE('Add AllChannels group');
        $this->groups->put(
            new AllChannelsGroup(
                $this,
                LanetConfig::ALL_CHANNEL_GROUP_ID,
                LanetConfig::ALL_CHANNEL_GROUP_CAPTION,
                LanetConfig::ALL_CHANNEL_GROUP_ICON_PATH));

        foreach ($xml->tv_categories->children() as $xml_tv_category)
        {
            if ($xml_tv_category->getName() !== 'tv_category')
            {
                TRACE("Error: unexpected node '" . $xml_tv_category->getName() .
                    "'. Expected: 'tv_category'");
                throw new Exception('Invalid XML document');
            }

            $group = new LanetGroup(
                strval($xml_tv_category->id),
                strval($xml_tv_category->caption),
                strval($xml_tv_category->icon_url));
            
            $group->trace_data();
            
            $this->groups->put($group);
        }

        foreach ($xml->tv_channels->children() as $xml_tv_channel)
        {
            if ($xml_tv_channel->getName() !== 'tv_channel')
            {
                hd_print("Error: unexpected node '" . $xml_tv_channel->getName().
                    "'. Expected: 'tv_channel'");
                throw new Exception('Invalid XML document');
            }

            $channel =
                new LanetChannel(
                    strval($xml_tv_channel->id),
                    strval($xml_tv_channel->caption),
                    strval($xml_tv_channel->icon_url),
                    strval($xml_tv_channel->streaming_url),
                    intval($xml_tv_channel->number),
                    intval($xml_tv_channel->num_past_epg_days),
                    intval($xml_tv_channel->num_future_epg_days));

            $this->channels->put($channel);
            $channel->trace_data();

            foreach ($xml_tv_channel->tv_categories->children() as $xml_tv_cat_id)
            {
                if ($xml_tv_cat_id->getName() !== 'tv_category_id')
                {
                    hd_print("Error: unexpected node '" . $xml_tv_cat_id->getName() .
                        "'. Expected: 'tv_category_id'");
                    throw new Exception('Invalid XML document');
                }

                $tv_category_id = intval($xml_tv_cat_id);

                $group = $this->groups->get($tv_category_id);

                // Link group and channel.

                $channel->add_group($group);
                $group->add_channel($channel);
            }
        }
		
        // dune stucks if any group has an emply channels list
        TRACE('cleanup empty channels:');

        $new_groups = new HashedArray();
        foreach($this->groups as $value)
        {
            if (($value->get_id() ===  LanetConfig::ALL_CHANNEL_GROUP_ID) ||
                ($value->get_id() === LanetConfig::FAV_CHANNEL_GROUP_ID))
            {
                TRACE('  skip '.$value->get_id());
                $new_groups->put($value);
                continue;
            }

            $group_channels = $value->get_channels($plugin_cookies);
            TRACE('  group:'.$value->get_title().' has ch:'.$group_channels->size());

            if ($group_channels->size() !==  0)
            {
                $new_groups->put($value);
            }
            else
            {
                TRACE('  Empty list!');
            }
        }		

        $this->groups = $new_groups;
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies)
    {
        TRACE('get_tv_info:');
        TRACE('  media_url:');
        foreach($media_url as $key => $value)
        {
            TRACE('    '.$key.'->'.$value);
        }
        TRACE('  plugin_cookies:');
        foreach($plugin_cookies as $key => $value)
        {
            TRACE('    '.$key.'->'.$value);
        }
        
        return parent::get_tv_info($media_url, &$plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////
    
    public function get_tv_stream_url($media_url, &$plugin_cookies)
    { 
        TRACE('get_tv_stream_url:');
        TRACE('  media_url:'.$media_url);
        TRACE('  plugin_cookies:');
        foreach($plugin_cookies as $key => $value)
        {
            TRACE('    '.$key.'->'.$value);
        }
        
        $this->ensure_channels_loaded($plugin_cookies);

        return $media_url; //$this->get_channel($channel_id)->get_streaming_url();
    }
    
    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
    {
        //$this->ensure_channels_loaded($plugin_cookies);
        TRACE('get_tv_playback_url:');
        TRACE('  channel_id:'.$channel_id);
        TRACE('  archive_ts:'.$archive_ts);
        TRACE('  protect_code:'.$protect_code);
        TRACE('  plugin_cookies:');
        foreach($plugin_cookies as $key => $value)
        {
            TRACE('    '.$key.'->'.$value);
        }
        
        return parent::get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies);
    }
    
    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
    {
        TRACE('get_day_epg_iterator:');
        TRACE('  channel_id:'.$channel_id);
        TRACE('  day_start_ts:'.$day_start_ts);
        TRACE('  plugin_cookies:');
        foreach($plugin_cookies as $key => $value)
        {
            TRACE('    '.$key.'->'.$value);
        }
        
        if (LanetConfig::EPG_URL_FORMAT == null)
            return array();

        try
        {
            $doc =
                HD::http_get_document(//LanetConfig::EPG_URL_FORMAT);
                    sprintf(
                        LanetConfig::EPG_URL_FORMAT,
                        $channel_id,
                        $day_start_ts));
        }
        catch (Exception $e)
        {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }
        
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print( "XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'epg')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'NewDataSet'");
            throw new Exception('Invalid XML document');
        }

        $epg = array();

        foreach ($xml->children() as $epg_item)
        {
            if ($epg_item->getName() == 'channel_id')
            {
                hd_print('Received epg for channel: '.$epg_item);
                continue;
            }
            
            if ($epg_item->getName() !== 'epg_item')
            {
                hd_print("Error: unexpected node '" . $epg_item->getName() .
                    "'. Expected: 'epg_item'");
                throw new Exception('Invalid XML document');
            }

            $epg[] =
                new DefaultEpgItem(
                    strval($epg_item->caption),
                    strval($epg_item->description),
                    intval($epg_item->start_time),
                    intval($epg_item->stop_time));
        }
        
        TRACE('get EGP between');
        TRACE('  '.$day_start_ts.' - '.HD::format_date_time_date($day_start_ts).' '.HD::format_date_time_time($day_start_ts));
        TRACE('  '.($day_start_ts+86400).' - '.HD::format_date_time_date($day_start_ts+86400).' '.HD::format_date_time_time($day_start_ts+86400));
        
        return
            new EpgIterator(
                $epg,
                $day_start_ts,
                $day_start_ts + 86400); // + 24 hours
    }

    public function folder_entered(MediaURL $media_url, &$plugin_cookies)
    {
        TRACE('folder_entered:');
        TRACE('  media_url:');
        foreach($media_url as $key => $value)
        {
            TRACE('    '.$key.'->'.$value);
        }
        TRACE('  plugin_cookies:');
        foreach($plugin_cookies as $key => $value)
        {
            TRACE('    '.$key.'->'.$value);
        }
        
    }
}

///////////////////////////////////////////////////////////////////////////
?>
